/*
Example of adding records to a database using the C/Database Toolchest for
database management and the C/Windows Toolchest for user interface.  This
example assumes a fixed database format.  It also illustrates the use of
an enumeration type to index the database fields.
*/

#include "stdlib.h"
#include "string.h"
#include "isam.h"
#include "cbtree.h"
#include "winlib.h"
#if ! defined(OK)
   #define OK I_OK
#endif

/* List of fields in the database */
char *field_names[] = {
   "Last Name", "First Name", "Address", "City",
   "State", "Zip", "Phone", NULL};
enum field_id {db_last, db_first, db_address, db_city,
               db_state, db_zip, db_phone};
#define NFIELDS (sizeof(field_names)/sizeof(char *)-1)
int num_fields = NFIELDS;

void error(Window wd, char *msg1, char *msg2);

main()
{
   Window info_window;     /* window to display status */
   Window edit_window;     /* window for editing a record */
   char name[50];          /* name of the database */
   char bufr[50];          /* field prompt buffer */
   int status, key, len;
   enum field_id id;
   char **db_field_names;
   Field fname;
   Field fld[NFIELDS];
   char record[NFIELDS][50];
   char *ptrs[NFIELDS];
   Db_Obj *db;             /* the database */

   info_window = w_open(20, 2, 40, 2);
   edit_window = w_open(10, 5, 60, 7);
   if (info_window == NULL || edit_window == NULL) {w_closeall(); return -1;}

   /* use a field to get the name of the database */
   fname = f_create("Database name: ","___________________");
   status = f_process(info_window, 1, 0, fname);
   if (status == -1) error(info_window, "Can't get name of database",NULL);
   if (status == _ESC) {w_closeall(); return 1;}
   f_getstring(fname, name);
   f_free(fname);

   /* open the database and check its format */
   db = iopen_db(name);
   if (db == NULL) {
      w_putsat(info_window, 0, 1, "Database not found, create it?");
      key = k_getkey();
      if (key != 'Y' && key != 'y') {
         w_closeall();
         return 1;
         }
      db = icreate_db(name, DEFAULT_BLKSIZE, field_names);
      if (db == NULL) error(info_window, "Unable to create:", name);
      }
   if (igetfldcount(db) != num_fields)
      error(info_window, "Number of fields does not match", name);
   if ((db_field_names = igetfldnames(db)) == NULL)
      error(info_window, "Unable to get field names", name);
   for (id = db_last; id <= db_phone; id++) {
      if (strcmp(*db_field_names, field_names[id]) != 0)
         error(info_window, "Field names do not match", name);
      db_field_names++;
      }

   /* create a data entry field for each database field */
   for (id = db_last; id <= db_phone; id++) {
      strcpy(bufr, field_names[id]);     /* make prompts all the same size */
      len = strlen(bufr);
      if (len < 10) memset(&bufr[len], ' ', 10-len);
      memcpy(&bufr[10], ": ", 3);
      fld[id] = f_create(bufr, "________________________________________");
      if (fld[id] == NULL) error(info_window, "Can't create fields", NULL);
      if (f_append(edit_window, 0, id, fld[id]) != 0)
         error(info_window, "Can't add field to window", NULL);
      }

   do {     /* input data until escape */
      f_clearall(edit_window);
      key = f_batch(edit_window);
      if (key != _ESC) {
         for (id = db_last; id <= db_phone; id++) {
            f_getstring(fld[id], record[id]);
            ptrs[id] = record[id];
            }
         if (iaddrec(db, NULL, ptrs) != OK)
            error(info_window,"Error adding record", NULL);
         }
      } while (key != _ESC);

   iclose_db(db);
   w_closeall();
   }

void error(Window wd, char *msg1, char *msg2)
{
   w_clear(wd);
   w_poscurs(wd, 0, 0);
   if (msg1 != NULL) w_printl(wd,msg1);
   if (msg2 != NULL) w_print(wd,msg2);
   k_getkey();
   w_closeall();
   isamexit();
   exit(-1);
   }

