/*
Simple example of editing a database record using the C/Database Toolchest
and C/Windows Toolchest.  Requests the name of the database and then edits
records starting at the first one in the first index.  The Page Up and
Page Down keys move between records.  Esc exits from the program.
*/

#include "stdlib.h"
#include "db_edit.c"
void error(Window wd, char *msg1, char *msg2);
unsigned termkeys[4] = {_ESC, _PGUP, _PGDN, 0};

main()
{
   Window info_window;     /* window to display status */
   Window edit_window;     /* window for editing a record */
   char name[50];          /* name of the database */
   char **index_names;     /* names of indexes */
   int status, key;
   Field fname;
   Db_Obj *db;             /* the database */
   Index_Obj *idx;         /* current index */

   info_window = w_open(20, 2, 40, 2);
   edit_window = w_open(10, 5, 60, 16);
   if (info_window == NULL || edit_window == NULL) return -1;

   /* use a field to get the name of the database */
   fname = f_create("Database to edit: ","___________________");
   status = f_process(info_window, 1, 0, fname);
   if (status == -1) error(info_window, "Can't get name of database",NULL);
   if (status == _ESC) {w_closeall(); return 1;}
   f_getstring(fname, name);
   f_free(fname);

   /* open the database and find the first index */
   db = iopen_db(name);
   if (db == NULL) error(info_window, "Database not found", name);
   index_names = igetindexnames(db);
   if (index_names == NULL) error(info_window, "No index names", name);
   if (strcmp(*index_names, iphysical)==0 && *(index_names+1) != NULL)
      index_names++;
   idx = ihandle(db, *index_names);
   if (idx == NULL) error(info_window, "Can't open index", *index_names);
   status = ifindhead(db, idx);

   f_termkeys(termkeys);   /* exit on Esc, PageUp, or PageDown */
   w_putsat(info_window, 0, 1, "use Esc to exit, PgUp & PgDown to move");

   /* edit records */
   do {
      key = db_edit_record(edit_window, db, idx);
      if (key == _PGUP) status = ifindprev(db, idx);
      if (key == _PGDN) status = ifindnext(db, idx);
      } while (status == OK && key != _ESC);

   iclose_db(db);
   w_closeall();
   }

void error(Window wd, char *msg1, char *msg2)
{
   w_clear(wd);
   w_poscurs(wd, 0, 0);
   if (msg1 != NULL) w_printl(wd,msg1);
   if (msg2 != NULL) w_print(wd,msg2);
   k_getkey();
   w_closeall();
   isamexit();
   exit(-1);
   }

